@@ -1,13 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2021 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2022 -- leonerd@leonerd.org.uk
 
 use v5.26; # signatures
-use Object::Pad 0.57;
+use warnings;
+use Object::Pad 0.73 ':experimental(adjust_params init_expr)';
 
-package Tickit::Widget::Scroller 0.28;
+package Tickit::Widget::Scroller 0.30;
 class Tickit::Widget::Scroller
+   :strict(params)
    :isa(Tickit::Widget);
 
 use Tickit::Style;
@@ -70,9 +72,17 @@ display
 =cut
 
 style_definition base =>
-   indicator_rv => 1;
+   indicator_rv => 1,
+   "<Down>"     => "scroll_down_line",
+   "<Up>"       => "scroll_up_line",
+   "<PageDown>" => "scroll_down_halfpage",
+   "<PageUp>"   => "scroll_up_halfpage",
+   "<C-Home>"   => "scroll_to_top",
+   "<C-End>"    => "scroll_to_bottom",
+   ;
 
 use constant WIDGET_PEN_FROM_STYLE => 1;
+use constant KEYPRESSES_FROM_STYLE => 1;
 
 =head1 KEYBINDINGS
 
@@ -140,11 +150,11 @@ C<set_gen_top_indicator> and C<set_gen_bottom_indicator>.
 
 =cut
 
-has @_items;
-has @_itemheights;
+field @_items;
+field @_itemheights;
 
-has $_start_item = 0;
-has $_start_partial = 0;
+field $_start_item = 0;
+field $_start_partial = 0;
 
 # accessor methods for t/30indicator.t to use
 # TODO: Should think about whether these should be made public
@@ -154,22 +164,21 @@ method _start_partial { $_start_partial }
 
 # We're going to cache window height because we need pre-resize height
 # during resize event
-has $_window_lines;
-has $_window_cols;
+field $_window_lines;
+field $_window_cols;
 
-has $_gravity_bottom;
+field $_gravity_bottom;
 
-has $_pending_scroll_to_bottom;
+field $_pending_scroll_to_bottom;
 
-has $_on_scrolled :param :reader :writer = undef;
+field $_on_scrolled :param :reader :writer = undef;
 
-has $_gen_top_indicator    :param = undef;
-has $_gen_bottom_indicator :param = undef;
-
-ADJUSTPARAMS ( $params )
-{
-   my $gravity = ( delete $params->{gravity} ) || "top";
+field $_gen_top_indicator    :param = undef;
+field $_gen_bottom_indicator :param = undef;
 
+ADJUST :params (
+   :$gravity = "top",
+) {
    $_gravity_bottom = ( $gravity eq "bottom" );
 }
 
@@ -902,26 +911,17 @@ method render_to_rb ( $rb, $rect )
    }
 }
 
-my %bindings = (
-   Down => sub { $_[0]->scroll( +1 ) },
-   Up   => sub { $_[0]->scroll( -1 ) },
+method key_scroll_to_top    { $self->scroll_to_top }
+method key_scroll_to_bottom { $self->scroll_to_bottom }
 
-   PageDown => sub { $_[0]->scroll( +int( $_[0]->window->lines / 2 ) ) },
-   PageUp   => sub { $_[0]->scroll( -int( $_[0]->window->lines / 2 ) ) },
+method key_scroll_up_line   { $self->scroll( -1 ) }
+method key_scroll_down_line { $self->scroll( +1 ) }
 
-   'C-Home' => sub { $_[0]->scroll_to_top },
-   'C-End'  => sub { $_[0]->scroll_to_bottom },
-);
+method key_scroll_up_halfpage   { $self->scroll( -int( $self->window->lines / 2 ) ) }
+method key_scroll_down_halfpage { $self->scroll( +int( $self->window->lines / 2 ) ) }
 
-method on_key ( $ev )
-{
-   if( $ev->type eq "key" and my $code = $bindings{$ev->str} ) {
-      $code->( $self );
-      return 1;
-   }
-
-   return 0;
-}
+method key_scroll_up_page   { $self->scroll( $self->window->lines ) }
+method key_scroll_down_page { $self->scroll( $self->window->lines ) }
 
 method on_mouse ( $ev )
 {
@@ -978,8 +978,8 @@ return different text now.
 
 =cut
 
-has %_indicator_win;
-has %_indicator_text;
+field %_indicator_win;
+field %_indicator_text;
 
 method update_indicators ()
 {
@@ -1035,10 +1035,6 @@ method update_indicators ()
 Abstract away the "item storage model" out of the actual widget. Implement
 more storage models, such as database-driven ones.. more dynamic.
 
-=item *
-
-Keybindings
-
 =back
 
 =cut
